package com.nickmacholl.minecraft.ChainWorld;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import org.bukkit.Bukkit;
import org.bukkit.entity.Player;
import org.bukkit.plugin.java.JavaPlugin;
import org.bukkit.scheduler.BukkitScheduler;

import com.nickmacholl.minecraft.ChainWorld.events.PlayerDeathListener;
import com.nickmacholl.minecraft.ChainWorld.events.PlayerJoinListener;
import com.nickmacholl.minecraft.ChainWorld.events.PlayerLoginListener;
import com.nickmacholl.minecraft.ChainWorld.events.ServerListPingListener;
import com.nickmacholl.minecraft.ChainWorld.io.ymlFileIO;

public final class ChainWorld extends JavaPlugin{
	
	private ymlFileIO lease;
	private ymlFileIO queue;
	private ymlFileIO emailregistry;
	private List<String> tempList = new ArrayList<String>();
	
	final ChainWorld plugin = this;
	final boolean dynDiff = getConfig().getBoolean("dynamicDifficulty");
	
	@Override
	public void onEnable(){
		//Setup external files: config.yml, lease.yml and queue.yml
		saveDefaultConfig();
		lease = new ymlFileIO(this.getDataFolder().toString()+"/lease.yml");
		queue = new ymlFileIO(this.getDataFolder().toString()+"/queue.yml");
		emailregistry = new ymlFileIO(this.getDataFolder().toString()+"/emailregistry.yml");
		
	    getServer().getPluginManager().registerEvents(new PlayerLoginListener(this), this);
	    getServer().getPluginManager().registerEvents(new PlayerDeathListener(this), this);
	    getServer().getPluginManager().registerEvents(new PlayerJoinListener(this), this);
	    if(getConfig().getBoolean("dynamicMOTD")){
	    	getServer().getPluginManager().registerEvents(new ServerListPingListener(this), this);
	    }
	    
	    //This establishes the recurring calls to the run() method.
	    BukkitScheduler scheduler = Bukkit.getServer().getScheduler();
	    	scheduler.scheduleSyncRepeatingTask(this, new Runnable() {
	    		@Override
	            public void run() {
	    			//This gets run every minute.
	    			LandLord l = new LandLord(plugin);
	    			Messager m = new Messager(plugin);
	    			try {
						l.checkLeaseStatus();
					} catch (Exception e) {
						m.sendConsoleError(e.toString());
					}
	    			//Adjust the difficulty based upon the time elapsed.
	    			if(l.getLeaseState()==1 && dynDiff){
		    			GameMaster gm = new GameMaster(plugin);
	    				gm.moderateDifficulty(l.getHoursLeft());
	    			}
	            }
	        }, 0L, 1200L);
	    	
	    //Register the plugin's commands.
		Commander commander = new Commander(this);
		this.getCommand("cwlease").setExecutor(commander);
		this.getCommand("cwquit").setExecutor(commander);
		this.getCommand("cwregister").setExecutor(commander);
		this.getCommand("cwunregister").setExecutor(commander);
	}
	
	@Override
	public void onDisable() {
		//Save the flat files stored in memory.
		lease.save();
		queue.save();
		emailregistry.save();
	}
	
	public boolean isEmailConfigured(){
		if(getConfig().isSet("username") && getConfig().isSet("password")
				&& getConfig().isSet("auth") && getConfig().isSet("starttls")
				&& getConfig().isSet("host") && getConfig().isSet("port")){
			return true;
		}
		
		return false;
	}
	
	//If the server.properties doesn't have the IP address we will retrieve it
	//from the outside using amazon services.
	public String getServerIP() throws Exception{
		String ip = Bukkit.getServer().getIp();
		if(ip.length()==0){
			Messager m = new Messager(this);
			try{
				URL whatismyip = new URL("http://checkip.amazonaws.com");
				BufferedReader in = new BufferedReader(new InputStreamReader(
				                whatismyip.openStream()));
	
				ip = in.readLine(); //you get the IP as a String
			} catch (Exception e){
				m.sendConsoleError(e.toString());
			}
		}
		return ip;
	}
	
	public ymlFileIO getLeaseFile(){
		return lease;
	}
	
	public ymlFileIO getQueueFile(){
		return queue;
	}
	
	public ymlFileIO getEmailRegistry(){
		return emailregistry;
	}
	
	public boolean isInTempList(Player p){
		if(tempList.size()>0){
			return tempList.contains(p.getUniqueId().toString());
		}
		return false;
	}
	
	public void addPlayerToTempList(Player p){
		tempList.add(p.getUniqueId().toString());
	}
	
	public void removePlayerFromTempList(Player p){
		tempList.remove(p.getUniqueId().toString());
	}
}
