package com.nickmacholl.minecraft.ChainWorld;

import org.bukkit.Bukkit;
import org.bukkit.ChatColor;
import org.bukkit.command.Command;
import org.bukkit.command.CommandExecutor;
import org.bukkit.command.CommandSender;
import org.bukkit.command.ConsoleCommandSender;
import org.bukkit.entity.Player;

public class Commander implements CommandExecutor{
	
	public final ChainWorld plugin;
	
	public Commander(ChainWorld plugin){
		this.plugin = plugin;
	}
	
	public boolean onCommand(CommandSender sender, Command cmd, String label, String[] args){
		String command = cmd.getName().toLowerCase();
		Messager m = new Messager(plugin);
		
		//Command which returns the expiration date and remaining time on the current lease.
		if(command.equals("cwlease")){
			LandLord l = new LandLord(plugin);
			if(sender instanceof ConsoleCommandSender){
				int i = l.getLeaseState();
				switch (i){
				case 0:
					m.sendConsoleMessage("Waiting for player to accept the lease. " + l.getHoursLeft() + " hours until timeout.");
					break;
				case 1:
					m.sendConsoleMessage("The current lease expires on " + l.getExpirationDateString() + "; " + l.getHoursLeft() + " hours remain." );
					break;
				case 2:
					m.sendConsoleMessage("The server is awaiting a player.");
					break;
				default:
					m.sendConsoleError("Lease state error! This is catastrophic!");
					break;
				}
				//Send the console the expiration time and remaining hours.
				
			}else if(sender instanceof Player){
				m.sendPlayerMessage("Your lease expires on " + ChatColor.AQUA + l.getExpirationDateString() + ChatColor.RESET + "; " + ChatColor.AQUA + String.valueOf(l.getHoursLeft()) + ChatColor.RESET + " hours remain.", ((Player) sender).getPlayer());
				return true;
			}
		}
		
		//Player command that abandons the current lease.
		if(command.equals("cwquit")){
			LandLord l = new LandLord(plugin);
			
			Player[] pList = Bukkit.getOnlinePlayers();
			
			for(Player p : pList){
				if(!p.hasPermission("chainworld.admin")){
					p.kickPlayer("The lease was terminated by " + sender.getName() + ".");
				}
			}
			
			l.setExpirationDate(0);
			l.setLeaseState(0);
			l.cleanHouse(l.getOwnerID());
			try {
				l.checkLeaseStatus();
			} catch (Exception e) {
				m.sendConsoleError(e.toString());
			}
			
			m.sendConsoleMessage(sender.getName().toString() + " terminated the lease.");
			
			return true;
		}
		
		//Player command that registers player's emails.
		if(command.equals("cwregister")){
			Player p = (Player) sender;
			Mailer mailer = new Mailer(plugin);
			if(plugin.isEmailConfigured()){
				if(sender instanceof Player){
					if(args.length > 0){
						if(mailer.isValidEmail(args[0])){
							mailer.registerPlayer(p, args[0]);
							m.sendPlayerMessage("The email " + ChatColor.AQUA + args[0] + ChatColor.RESET + " was registered.\nYou will now receive emails when you are given a lease.", p);
						}else{
							m.sendPlayerMessage("Bad arguments. Try /cwregister example@email.com", p);
						}
					}else{
						m.sendPlayerMessage("Try /cwregister example@email.com", p);
					}
				}else{
					m.sendConsoleError("This command can only be run by players.");
				}
			}else{
				m.sendPlayerMessage("The server administrator hasn't configured email notifications.",p);
				return true;
			}
			return true;
		}
		
		//Player command that un-registers player's emails.
		if(command.equals("cwunregister")){
			Player p = (Player) sender;
			Mailer mailer = new Mailer(plugin);
			if(plugin.isEmailConfigured()){
				if(sender instanceof Player){
					if(mailer.isRegistered(p.getUniqueId().toString())){
						m.sendPlayerMessage("You un-registered " + ChatColor.AQUA + mailer.getEmail(p.getUniqueId().toString()) + ChatColor.RESET + ". You will no longer receive emails.", p);
						mailer.removeEmail(p);
					}else{
						m.sendPlayerMessage("You don't have an email registered.",p);
					}
				}else{
					m.sendConsoleError("This command can only be run by players.");
				}
				return true;
			}else{
				m.sendPlayerMessage("The server administrator hasn't configured email notifications.",p);
				return true;
			}
		}
		
		return false;
	}
}
