package com.nickmacholl.minecraft.ChainWorld;

import java.util.List;

import org.bukkit.Bukkit;
import org.bukkit.Difficulty;
import org.bukkit.World;
import org.bukkit.block.Block;

public class GameMaster {
	
	public final ChainWorld plugin;
	
	public GameMaster(ChainWorld plugin){
		this.plugin = plugin;
	}
	
	
	//This loops though all the worlds and changes their difficulty.
	public void setDifficulty(Difficulty d){
		List<World> worlds = Bukkit.getServer().getWorlds();
		for(World element : worlds){
			element.setDifficulty(d);
		}
	}
	
	//This loops though all the worlds and changes the monster spawn rate.
	//The change is a multiple of the server properties spawn rate.
	//This should increase as the time goes on.
	public void setMonsterSpawnLimit(double k){
		List<World> worlds = Bukkit.getServer().getWorlds();
		for(World element : worlds){
			int m = Bukkit.getMonsterSpawnLimit();
			element.setMonsterSpawnLimit((int)(k*m));
		}
	}
	
	//This loops though all the worlds and changes the animal spawn rate.
	//The change is a multiple of the server properties spawn rate.
	//This should decrease as the time goes on.
	public void setAnimalSpawnLimit(double k){
		List<World> worlds = Bukkit.getServer().getWorlds();
		for(World element : worlds){
			int m = Bukkit.getAnimalSpawnLimit();
			element.setAnimalSpawnLimit((int)(m*k));
		}
	}
	
	//Sets the time of day to dawn.
	public void setNewDay(){
		List<World> worlds = Bukkit.getServer().getWorlds();
		for(World element : worlds){
			element.setTime(6000);
		}
	}
	
	//Generate a new spawn for the worlds.
	public void newWorldSpawn(){
		
		//Get the overworld
		World overworld = Bukkit.getServer().getWorlds().get(0);
		
		//If the random spawn dimension is 0 treat the feature as disabled.
		int sDim = plugin.getConfig().getInt("spawnDimension");
		if(sDim==0){
			return;
		}
		
		boolean set = false;
		
		while(!set){
			int x = (int) (sDim - Math.random()*sDim);
			int z = (int) (sDim - Math.random()*sDim);
			int y = overworld.getMaxHeight();
			
			overworld.getChunkAt(x, z).load(true);
			
			while (y>overworld.getSeaLevel() && !set){
				y--;
				Block spawn = overworld.getBlockAt(x, y, z);
				if(!spawn.isEmpty() && !spawn.isLiquid()){
					overworld.setSpawnLocation(x, y+1, z);
					set = true;
				}
			}
		}
		
	}
	
	public void moderateDifficulty(double hours){
		//Set the appropriate difficulty for the time elapsed.
		int i = plugin.getConfig().getInt("leaseDuration");
		
		if(hours < i*(1/3)){
			setDifficulty(Difficulty.HARD);
		}else if(hours < i*(2/3)){
			setDifficulty(Difficulty.NORMAL);
		}else{
			setDifficulty(Difficulty.EASY);
		}
		
		//Change the monster/animal spawn limit
		double a = Math.sin((hours*Math.PI)/(2*i));
		double m = Math.cos((hours*Math.PI)/(2*i));
		
		setAnimalSpawnLimit(a);
		setMonsterSpawnLimit(m);
	}
}
