package com.nickmacholl.minecraft.ChainWorld;

import java.io.File;
import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import org.bukkit.Bukkit;
import org.bukkit.ChatColor;
import org.bukkit.entity.Player;

import com.nickmacholl.minecraft.ChainWorld.io.ymlFileIO;

public class LandLord {
	
	ymlFileIO lease;
	ymlFileIO queue;
	
	public final ChainWorld plugin;
	
	//The landlord needs his business papers!
	public LandLord(ChainWorld plugin){
		this.plugin = plugin;
		
		lease = plugin.getLeaseFile();
		lease.load();
		
		queue = plugin.getQueueFile();
		queue.load();
	}
	
	//Returns a human readable string of the expiration time.
	public long getExpirationTime(){
		long l = lease.getData().getLong("expires");
		return l;
	}
	
	//Get's the name of the lease owner.
	public String getOwnerID(){
		String s = lease.getData().getString("UUID");
		return s;
	}
	
	//Get the lease state.
	public int getLeaseState(){
		int i = lease.getData().getInt("state");
		return i;
	}
	
	//Get the queue list
	public List<String> getQueue(){
		queue.load();
		List<String> s = queue.getData().getStringList("waitlist");
		return s;
	}
	
	public boolean isInQueue(Player p){
		List<String> s = getQueue();
		if(s.size()>0){
			if(s.contains(p.getUniqueId().toString())){
				return true;
			}
		}
		return false;
	}
	
	//Get the next player in the queue's UUID
	public String getNextPlayerUUID(){
		List<String> t = getQueue();
		String s = null;
		if(t.size()>0){
			s = t.get(0);
		}
		return s;
	}
	
	//Get the player's place in line
	public int getQueuePosition(String uuid){
		List<String> s = getQueue();
		int x = s.indexOf(uuid)+1;
		return x;
	}
	
	//Get the queue size.
	public int getQueueSize(){
		List<String> s = getQueue();
		int x = s.size();
		return x;
	}
	
	//Remove the first element and shift it all up
	public void bumpQueue(){
		List<String> s = getQueue();
		s.remove(0);
		queue.getData().set("waitlist", s);
		queue.save();
	}
	
	//Add a player to the queue, provided they aren't in it already.
	public void addPlayerToQueue(Player p){
		List<String> t = getQueue();
		String s = p.getUniqueId().toString();
		if(!t.contains(s)){
			t.add(s);
		}
		queue.getData().set("waitlist", t);
		queue.save();
	}
	
	//Add a player to the queue.
	public void addPlayerToQueue(String uuid){
		List<String> s = getQueue();
		if(!s.contains(uuid)){
			s.add(uuid);
		}
		queue.getData().set("waitlist", s);
		queue.save();
	}
	
	//Is the lease expired? Simple boolean.
	public boolean isExpired(){
		Date d = new Date();
		long l = lease.getData().getLong("expires");
		
		if(d.getTime() < l){
			return false;
		}else{
			return true;
		}
	}
	
	//Set the name of the lease owner.
	public void setOwnerID(String id){
		lease.getData().set("UUID", id);
		lease.save();
	}
	
	//Set the expiration date of the lease.
	public void setExpirationDate(long exp){
		lease.getData().set("expires", exp);
		lease.save();
	}
	
	//Set lease state
	public void setLeaseState(int state){
		lease.getData().set("state", state);
		lease.save();
	}
		
	public void createLease(String uuid, int hours, int state, boolean sendEmail) throws Exception{
		setOwnerID(uuid);
		
		Date d = new Date();
		
		setExpirationDate(d.getTime() + 1000 * 60 * 60 * hours);
		
		setLeaseState(state);
		
		Mailer mail = new Mailer(plugin);
		if(mail.isRegistered(uuid) && sendEmail){
			mail.sendEmail(mail.getEmail(uuid), mail.getName(uuid));
		}
	}
	
	//Get remaining lease time in hours.
	public double getHoursLeft(){
		Date d = new Date();
		long now = d.getTime();
		long exp = getExpirationTime();
		double hours =  (double) (exp-now)/(1000 * 60 * 60);
		double n = Double.parseDouble(new DecimalFormat("#.#").format(hours));
		
		return n;
	}
	
	//Get remaining lease time in minutes.
	public int getMinutesLeft(){
		Date d = new Date();
		long now = d.getTime();
		long exp = getExpirationTime();
		int minutes =  (int) (exp-now)/(1000 * 60);
		
		return minutes;
	}
	
	public String getExpirationDateString(){
		Date d = new Date();
		Long ln = lease.getData().getLong("expires");
		DateFormat df = new SimpleDateFormat("MM/dd/yyyy HH:mm");
		d.setTime(ln);
		
		String s = df.format(d);
		
		return s;
	}
	
	//Delete the player data.
	public void cleanHouse(String uuid){
		//TODO More dynamic file grabbing.
		String w = Bukkit.getServer().getWorlds().get(0).getName();
		
        final File dat1 = new File(w + "/playerdata/" + uuid + ".dat");
        final File dat2 = new File(w + "_nether/playerdata/" + uuid + ".dat");
        final File dat3 = new File(w + "_the_end/playerdata/" + uuid + ".dat");
        
        //Set a new spawn location.
        GameMaster gm = new GameMaster(plugin);
        gm.newWorldSpawn();
        
        //Schedule the deletion of player data.
        Bukkit.getServer().getScheduler().scheduleSyncDelayedTask(plugin, new Runnable() {
              public void run() {
                  dat1.delete();
                  dat2.delete();
                  dat3.delete();
              }
            }, 20L);
	}
	
	//This gets called every minute to do some bookkeeping.
	public void checkLeaseStatus() throws Exception{
		Messager m = new Messager(plugin);
		
		int n = getMinutesLeft();
		//There is less than 1 hour remaining, notify the players (if there is anyone online.)
		if(n<=60 && !isExpired()){;
			if((n>=30) && (n % 10 == 0)){
				//Send a message every 10 minutes if more than 30 minutes remains.
				m.sendPlayerBroadcast("The current lease expires in " + n +" minutes." + ChatColor.RESET, ChatColor.WHITE);
			}else if((n>10) && (n<30) && (n % 3 == 0)){
				//Send a message every 3 minutes if more than 6 minutes remain, but less than 30.
				m.sendPlayerBroadcast("The current lease expires in " + n +" minutes." + ChatColor.RESET, ChatColor.YELLOW);
			}else if(n<=10){
				//Send a message every minute if less than 6 minutes remain.
				m.sendPlayerBroadcast("The current lease expires in " + n +" minutes!" + ChatColor.RESET, ChatColor.RED);
			}
		}
		
		if(isExpired()){
			switch (getLeaseState()){
			case 0:
				m.sendConsoleMessage("The current lease was abandoned.");
				break;
			case 1:
				m.sendConsoleMessage("The current lease has expired.");
				//Kick all the players on the server.
				Player[] pList = Bukkit.getOnlinePlayers();
				for(Player p : pList){
					if(!p.hasPermission("chainworld.admin")){
						p.kickPlayer(m.getKickMessage(4, p));
						cleanHouse(p.getUniqueId().toString());
					}
				}
				break;
			case 2:
				//The server is abandoned (do nothing).
				break;
			default:
				m.sendConsoleMessage("Something horrible happened.");
			}
			
			//Create a new "timeout" lease.
			if(getQueueSize()>0){
				createLease(getNextPlayerUUID(), plugin.getConfig().getInt("timeoutDuration"),0,true);
				bumpQueue();	
			}else{
				setLeaseState(2);
				setExpirationDate(0);
			}
		}
	}
}
