package com.nickmacholl.minecraft.ChainWorld;

import java.util.Properties;

import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.PasswordAuthentication;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.AddressException;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;

import org.bukkit.Bukkit;
import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.entity.Player;

public class Mailer {
	
	public final ChainWorld plugin;
	public final FileConfiguration registry;
	
	public Mailer(ChainWorld plugin){
		this.plugin = plugin;
		plugin.getEmailRegistry().load();
		registry = plugin.getEmailRegistry().getData();
	}
	
	public void registerPlayer(Player p, String email){
		registry.set(p.getUniqueId().toString()+".name", p.getName().toString());
		registry.set(p.getUniqueId().toString()+".email", email);
		plugin.getEmailRegistry().save();
	}
	
	public String getEmail(String uuid){
		return registry.getString(uuid+".email");
	}
	
	public String getName(String uuid){
		return registry.getString(uuid+".name");
	}
	
	public boolean isRegistered(String uuid){
		return registry.isSet(uuid+".email");
	}
	
	public void removeEmail(Player p){
		registry.set(p.getUniqueId().toString()+".email", null);
		plugin.getEmailRegistry().save();
	}
	
	public boolean isValidEmail(String email){
		if(email.length()>0 && email.contains("@") && email.contains(".")){
			try {
				InternetAddress emailAddr = new InternetAddress(email);
				emailAddr.validate();
			} catch (AddressException ex) {
				return false;
			}
			return true;
		}
		return false;
	}
	
    public void sendEmail(String email, String player) throws Exception{
    	//Import the necessary connection info for the SMTP server from config.yml.
    	final String username = plugin.getConfig().getString("username");
		final String password = plugin.getConfig().getString("password");
		final String auth = plugin.getConfig().getString("auth");
		final String starttls = plugin.getConfig().getString("starttls");
		final String host = plugin.getConfig().getString("host");
		final String port = plugin.getConfig().getString("port");
		//Import the lease details from config.yml
		final String timeOut = plugin.getConfig().getString("leaseTimeout");
		final String leaseTime = plugin.getConfig().getString("leaseDuration");
		//Get the server name and IP so the player can know what server they want.
		final String serverName = Bukkit.getServerName();
		final String serverIP = plugin.getServerIP();
		final String serverPort = String.valueOf(Bukkit.getPort());
 
		Properties props = new Properties();
		props.put("mail.smtp.auth", auth);
		props.put("mail.smtp.starttls.enable", starttls);
		props.put("mail.smtp.host", host);
		props.put("mail.smtp.port", port);
 
		Session session = Session.getInstance(props,
		  new javax.mail.Authenticator() {
			protected PasswordAuthentication getPasswordAuthentication() {
				return new PasswordAuthentication(username, password);
			}
		  });
 
		try {
 
			Message message = new MimeMessage(session);
			message.setFrom(new InternetAddress(username));
			message.setRecipients(Message.RecipientType.TO,
				InternetAddress.parse(email));
			message.setSubject(player + ", it's your turn to play!");
			//TODO Make the email message pull lines from config.yml
			message.setText(player + ","
			+ "\n\nYou have been assigned a " + leaseTime + " hour lease for the Minecraft chain world " + serverName + " at " + serverIP + ":" + serverPort + ". You have "
			+ timeOut + " hours to accept the lease or it will be forfeit to the next player in line. To accept the lease, all you have to do is login. "
			+ "\n\nGood luck and have fun!\n\n\nThis is an automated message. You could reply, but nobody is going to read it. If you don't want to get"
			+ " these emails just login and use the /cwunregister command.");
 
			Transport.send(message);
		} catch (MessagingException e) {
			throw new RuntimeException(e);
		}
    }
}