package com.nickmacholl.minecraft.ChainWorld.events;

import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.EventPriority;
import org.bukkit.event.Listener;
import org.bukkit.event.player.PlayerLoginEvent;
import org.bukkit.event.player.PlayerLoginEvent.Result;

import com.nickmacholl.minecraft.ChainWorld.GameMaster;
import com.nickmacholl.minecraft.ChainWorld.LandLord;
import com.nickmacholl.minecraft.ChainWorld.Messager;
import com.nickmacholl.minecraft.ChainWorld.ChainWorld;

public class PlayerLoginListener implements Listener{

	public final ChainWorld plugin;
	
	public PlayerLoginListener(ChainWorld plugin){
		this.plugin = plugin;
	}
	
	@EventHandler(priority = EventPriority.LOWEST)
    public void playerLogin(PlayerLoginEvent event) throws Exception {
		
		Player p = event.getPlayer();
		LandLord l = new LandLord(plugin);
		Messager m = new Messager(plugin);
		GameMaster g = new GameMaster(plugin);
		
		//Check if the player has admin privileges.
		if(!p.hasPermission("chainworld.admin")){
			//Is the lease expired?
			if(l.isExpired()){
				//Are there people waiting in line?
				if(l.getQueueSize()>0){
					//Give the next player in line a lease.
					l.createLease(l.getNextPlayerUUID(), plugin.getConfig().getInt("leaseTimeout"), 0,true);
					l.bumpQueue();
					//Does the connecting player happen to own the lease?
					if(p.getUniqueId().toString().equals(l.getOwnerID())){
						//Yes they are, let them in.
						event.allow();
						g.setNewDay();
					}else{
					//Do some bookkeeping with the connecting player.
						if(plugin.isInTempList(p)){
							//Add the player to the queue.
							l.addPlayerToQueue(p);
							plugin.removePlayerFromTempList(p);
							event.disallow(Result.KICK_OTHER, m.getKickMessage(1, p));
						}else{
							//Put the player in the temp list.
							//A reconnect attempt gets them on the actual queue.
							if(l.isInQueue(p)){
								event.disallow(Result.KICK_OTHER, m.getKickMessage(2, p));
							}else{
								plugin.addPlayerToTempList(p);
								event.disallow(Result.KICK_OTHER, m.getKickMessage(0, p));
							}
						}
					}
				}else{
					//There is nobody in line. Let the current player in!
					l.createLease(p.getUniqueId().toString(), plugin.getConfig().getInt("leaseDuration"), 1, false);
					g.setNewDay();
					event.allow();
				}
			}else{
				//The lease is not expired.
				if(p.getUniqueId().toString().equals(l.getOwnerID())){
					//The current player owns the lease.
					if(l.getLeaseState()==0){
						m.sendConsoleMessage(p.getName() + " has accepted their lease.");
						l.setLeaseState(1);
						g.setNewDay();
					}
					event.allow();
				}else{
					//The current player doesn't own the lease.
					if(plugin.isInTempList(p)){
						//Add the player to the queue.
						l.addPlayerToQueue(p);
						plugin.removePlayerFromTempList(p);
						event.disallow(Result.KICK_OTHER, m.getKickMessage(1, p));
					}else{
						//Put the player in the temp list.
						//A reconnect attempt gets them on the actual queue.
						if(l.isInQueue(p)){
							event.disallow(Result.KICK_OTHER, m.getKickMessage(2, p));
						}else{
							plugin.addPlayerToTempList(p);
							event.disallow(Result.KICK_OTHER, m.getKickMessage(0, p));
						}
					}
				}
			}
		}
	}
}
